from pathlib import Path

import pandoc
from docutils import nodes
from docutils.core import publish_doctree
from robot.testdoc import JsonConverter, TestSuiteFactory
from sphinx.util.docutils import SphinxDirective


class RfTestDoc(SphinxDirective):
    """A directive for RF TestDocs!"""
    required_arguments = 1

    def get_suites(self, suites: dict, ret: list):
        title = nodes.title(text=suites.get('name'))
        ret.append(title)
        if suites.get('doc'):
            doc_pan = pandoc.read(source=suites.get('doc'), format="html")
            doc_rst = pandoc.write(doc=doc_pan, format="rst")
            doc = list(publish_doctree(doc_rst))
            ret.extend(doc)
        if suites.get('keywords'):
            kw_lines = {
                'SETUP': [],
                'KEYWORD': [],
                'TEARDOWN': [],
            }
            for kw in suites.get('keywords'):
                kw_type = kw.get('type')
                kw_args = "    ".join(kw.get('arguments').split(','))
                kw_doc_line = f"{kw.get('name')}    {kw_args}"
                kw_lines[kw_type].append(kw_doc_line)

            for kw_type, lines in kw_lines.items():
                if lines:
                    kw_title = nodes.strong(text=f"{kw_type.title()}:")
                    ret.append(kw_title)
                    kw_doc = nodes.literal_block(text="\n".join(lines))
                    ret.append(kw_doc)

        for key, value in suites.items():
            if key == "suites" or key == "tests":
                for child_suite in value:
                    self.get_suites(child_suite, ret)

    def run(self) -> list[nodes.Node]:
        suite_root = Path(self.config.rf_test_doc_root) / self.arguments[0]
        raw_suite = TestSuiteFactory([suite_root])
        suites = JsonConverter("TestDoc").convert(raw_suite)
        ret = []
        self.get_suites(suites, ret)
        return ret
